#ifndef BITMAP
#define BITMAP

#include "BitmapCouleurs.h"

struct espace {
  int             dx ;
  int             dy ;
  int             bits ;
  int             l ;
  unsigned char   **p ; } ;

union pixel {
  struct c24bits     c ;
  unsigned char      t[3] ; } ;

unsigned char   new_espace(struct espace *e,int dx,int dy,int bits) ;
void            cherche_ligne(int y,unsigned char *l,struct espace *e) ;
void            sauve_ligne(int y,unsigned char *l,struct espace *e) ;
void            liberation_espace(struct espace *e) ;
void            init_espace(struct espace *e,unsigned char v) ;

#endif
